import pandas as pd
import numpy as np
import os
import argparse

# word 2 vec
from gensim.models import word2vec as w2v

class EmpiricalRarefaction(object):
    def __init__(self, mdl, function):
        self.mdl = w2v.Word2Vec.load(mdl)
        self.preds =  pd.read_pickle(os.path.join(os.path.dirname(mdl),
                                                                    'predictions/hypothetical_predictions.pkl'))
        self.function = function
        self.t_weighted = 0.9
        self.t_unweighted = 0.99
        self.out_dir = os.path.join(os.path.dirname(mdl),'predictions')

    def set_preds_to_function(self):
        preds = self.preds
        preds = preds[(preds['predicted_class'] == self.function) &
                      ((preds['weighted_prediction_score'] > self.t_weighted) |
                       (preds['predicted_class_score'] > self.t_unweighted))]
        self.preds = preds

    def bootstrap_samples(self, n_bootstraps, n_genes_min, n_genes_max, step, alpha=0.05):
        preds = self.preds
        mdl = self.mdl
        preds['word_count'] = preds['word'].apply(lambda w: mdl.wv.vocab[w].count)
        preds = preds[['word', 'word_count']]
        preds['words_by_count'] = preds.apply(lambda row: [row["word"]] * row['word_count'], axis=1)

        gene_words = preds['words_by_count'].explode().values

        res = []
        for n_genes in np.arange(n_genes_min, n_genes_max, step):
            n_genes = min(n_genes, len(gene_words))
            X = np.random.choice(gene_words, size=[n_bootstraps, n_genes])
            X_sorted = np.sort(X, axis=1)
            uniq_genes_dist = (X_sorted[:,1:] != X_sorted[:,:-1]).sum(axis=1)+1
            uniq_genes_mean = np.mean(uniq_genes_dist)
            upper_std = uniq_genes_mean + np.std(uniq_genes_dist)
            lower_std = uniq_genes_mean - np.std(uniq_genes_dist)
            upper_q = 2*uniq_genes_mean - np.quantile(uniq_genes_dist, alpha/2)
            lower_q = 2*uniq_genes_mean - np.quantile(uniq_genes_dist, 1 - alpha / 2)
            res.append((uniq_genes_mean, upper_std, lower_std, upper_q, lower_q, self.function, n_genes))
        df = pd.DataFrame(res, columns=['uniq_genes_mean', 'upper_std', 'lower_std',
                                        'upper_q', 'lower_q', 'function', 'n_genes'])

        df.to_pickle(os.path.join(self.out_dir, f'{self.function}_bootstrap.pkl'))
        return df




if __name__ == "__main__":
    argparse = argparse.ArgumentParser()
    argparse.add_argument('--model', required=True, type=str, help='model file path')
    argparse.add_argument('--function', required=True, type=str, help='functional group')
    argparse.add_argument('--min_genes', default=100, type=int, help='min number of genes to sample')
    argparse.add_argument('--max_genes', default=500000, type=int, help='max number of genes to sample')
    argparse.add_argument('--bootstrap', default=10000, type=int, help='mumber of bootstraps')
    argparse.add_argument('--step', default=1000, type=int, help='step size')
    params = argparse.parse_args()
    
    rarefaction = EmpiricalRarefaction(mdl=params.model, function=params.function)
    rarefaction.set_preds_to_function()
    rarefaction.bootstrap_samples(n_bootstraps=params.bootstrap, n_genes_min=params.min_genes, n_genes_max=params.max_genes, step=params.step, alpha=0.05)
    



import os
from tqdm import tqdm
import numpy as np
import pandas as pd
import glob
from Bio import SeqIO
import pickle
from BCBio import GFF

class Gff(object):
    def __init__(self, gff_path=None, hypothetical_folder='/hypothetical_mapping/',
                 ko_path="metadata.csv"):
        self.gff = gff_path
        self.ko_path = ko_path
        self.hypothetical = None
        self.keggRun = False
        self.hypotheticalRun = False
        self.clusterRun = False
        self.name = None
        self.gff_table = None

        self.fasta = self.gff.replace('.kg.05_21.gff', '.fa')
        self.proteins_fasta = self.gff.replace('.kg.05_21.gff', '.proteins.faa')
        self.hypothetical_path = os.path.join(hypothetical_folder, f"{os.path.basename(gff_path).split('.contig')[0]}.pkl")

    def __repr__(self):
        return self.name


    def set_name(self):
        # get the name of the file as the name of the gff object (includes some previous folder hierarchy)
        protein_fasta_path = self.proteins_fasta
        self.name = os.path.basename(protein_fasta_path).split(".contig")[0]

    def set_hypothetical(self):
        hypothetical_path = self.hypothetical_path
        if not os.path.exists(hypothetical_path):
            raise FileNotFoundError
        with open(hypothetical_path, 'rb') as hanlde:
            hypothetical_mapping = pickle.load(hanlde)
        self.hypothetical = hypothetical_mapping


    def set_gff_table(self):
        gff_table_path = f"{self.gff}.parsed.tsv"
        if not os.path.exists(gff_table_path):
            raise FileNotFoundError
        gff_table = pd.read_table(gff_table_path)
        self.gff_table = gff_table


    def parse_gff(self):
        f = self.gff
        if not os.path.exists(f):
            raise Exception("No GFF is currently available, need to run_kegg_qprokka first")
        if os.path.exists(f.replace(".gff", ".gff.parsed.tsv")):
            return
        
        records = []
        in_handle = open(f)
        for rec in tqdm(GFF.parse(in_handle)):
            for feature in rec.features:
                strand = feature.strand
                start = int(feature.location.start)
                end = int(feature.location.end)
                if feature.id == '' and 'locus_tag' in feature.qualifiers:
                    feature.id = feature.qualifiers['locus_tag'][0]#tmp addition due to curr gff file formats
                if "inference" not in feature.qualifiers:
                    feature.qualifiers["inference"] = "no inference record"

                if "product" in feature.qualifiers:
                    records.append((feature.id, feature.qualifiers['product'][0], feature.qualifiers['inference'][-1], feature.type, strand, start, end))
                else:
                    records.append((feature.id, feature.type, feature.type, feature.type, strand, start, end))
        in_handle.close()
        df = pd.DataFrame(records, columns=["contig_id", "product", "inference", "type", "strand", "start", "end"])
        df["annotation"] = df.apply(lambda row: _annotate(row["product"], row["inference"], row["type"]), axis=1)
        df["annotation_extended"] = df.apply(lambda row: _annotate_extended(row["product"], row["inference"],
                                                                            row["type"]), axis=1)

        output_path = f.replace(".gff", ".gff.parsed.tsv")
        df.to_csv(output_path, index=False, sep='\t')

    def extract_hypothetical(self):
        """ extract all hypothetical proteins for sequence-based clustering """
        protein_fasta_path = self.proteins_fasta
        hypothetical_fasta_path = protein_fasta_path.replace(".faa", ".hypothetical.faa")
        gff_table_path = self.gff.replace(".gff", ".gff.parsed.tsv")

        if self.hypotheticalRun or os.path.exists(hypothetical_fasta_path):
            self.hypotheticalRun = True
            return

        if not os.path.exists(gff_table_path):
            raise Exception("no parsed gff table was found - please run parse_gff to obtain the table")

        annotations = pd.read_table(gff_table_path)
        #adjust to lower case:
        annotations["annotation"] = annotations["annotation"].apply(lambda x: x.lower())
        hypothetical_ids = annotations[annotations["annotation"].isin(["hypothetical protein", "putative protein"])][
            "contig_id"].values
        hypothetical_records = []
        for rec in SeqIO.parse(protein_fasta_path, 'fasta'):
            if rec.name in hypothetical_ids:
                hypothetical_records.append(rec)
        with open(hypothetical_fasta_path, 'w') as handle:
            SeqIO.write(hypothetical_records, handle, "fasta")

        self.hypotheticalRun = True

    def extract_hypothetical_and_prokka(self):
        """ extract all hypothetical proteins for sequence-based clustering """
        protein_fasta_path = self.proteins_fasta
        hypothetical_fasta_path = protein_fasta_path.replace(".faa", ".hypothetical.prokka.faa")
        gff_table_path = self.gff.replace(".gff", ".gff.parsed.tsv")
        if os.path.exists(hypothetical_fasta_path):
            self.hypotheticalRun = True
            return

        if not os.path.exists(gff_table_path):
            raise Exception("no parsed gff table was found - please run parse_gff to obtain the table")

        annotations = pd.read_table(gff_table_path)
        ko_table = pd.read_table(self.ko_path)
        filter_ids = \
        annotations[(~annotations["annotation"].isin(ko_table['KO'])) & (annotations["type"] == 'CDS')][
            "contig_id"].values.tolist()

        hypothetical_and_prokka_records = []
        for rec in SeqIO.parse(protein_fasta_path, 'fasta'):
            if rec.name in filter_ids:
                hypothetical_and_prokka_records.append(rec)
        with open(hypothetical_fasta_path, 'w') as handle:
            SeqIO.write(hypothetical_and_prokka_records, handle, "fasta")
        self.hypotheticalRun = True
    # for in house use
    def cluster_hypothetical(self, queue="dudulight", threads=4):
        """ run mmseq cluster to generate a table of clustered """
        hypothetical_path = self.proteins_fasta.replace(".faa", ".hypothetical.faa")
        os.system(f"python /davidb/daniellemiller/bioutils/scripts/mmseq_cluster_runner.py --query {hypothetical_path} "
                  f"--queue {queue} --threads {threads}")
        self.hypotheticalRun = True

    def cluster_hypothetical_and_prokka(self, queue="dudulight", threads=4):
        """ run mmseq cluster to generate a table of clustered """
        hypothetical_path = self.proteins_fasta.replace(".faa", ".hypothetical.prokka.faa")
        os.system(f"python /davidb/daniellemiller/bioutils/scripts/mmseq_cluster_runner.py --query {hypothetical_path} "
                  f"--queue {queue} --threads {threads}")
        self.hypotheticalRun = True

    def assign_clusters(self):
        clustering_path = self.hypothetical_path
        if not os.path.exists(clustering_path):
            print("Clustering tsv do not exists, check whether cluster_hypothetical was previously run")
            return
        output_path = clustering_path.replace(".tsv", ".assigned.tsv")
        if os.path.exists(output_path):
            return pd.read_table(output_path)
        table = pd.read_table(clustering_path, names=["id1","id2"]) #id1 is the cluster representative, id2 is the matched orf
        table["cluster_id"] = pd.factorize(table["id1"])[0]
        table["cluster_id"] = table["cluster_id"].apply(
            lambda x: f"{os.path.basename(clustering_path).split('.contig')[0].replace('.', '_')}_Cluster_{x}")
        table.to_csv(output_path, index=False, sep='\t')
        return table


def _annotate(product, inference, gff_type):
    if "kg.05_21.ren4prok" in inference:
        return inference.split(":")[-1].split('.')[0]
    elif gff_type != "CDS":
        return gff_type
    else:
        return product

def _annotate_extended(product, inference, gff_type):
    if "kg.05_21.ren4prok" in inference:
        return inference.split(":")[-1]
    elif gff_type != "CDS":
        return gff_type
    else:
        return product














import pandas as pd
import re
import numpy as np
import os
import glob

from src.genomic_embeddings.models import NNClf
from src.genomic_embeddings.data import Embedding
import argparse
import pickle

argparse = argparse.ArgumentParser()
argparse.add_argument('--model', required=True, type=str, help='model file')
argparse.add_argument('--output',
                          default='predictions',
                          type=str, help='predictions output dir')
argparse.add_argument('--metadata',
                          default='metadata.csv',
                          type=str, help='metadata csv file path')
params = argparse.parse_args()

MODEL = params.model
METADATA = params.metadata
OUTPUT_DIR = params.output

label='label'


with open(glob.glob(os.path.join(os.path.dirname(MODEL), "predictions/label/FOLD_NO-PUMPS-CURATED-LABELS/*.pkl"))[0], 'rb') as o:
    class_2_aupr = pickle.load(o)
class_2_aupr = {k:v for k, v in class_2_aupr.items() if k != 'ALL'}
curated_labels = [k for k in class_2_aupr if k != 'ALL' and k != 'Other']

emb = Embedding(mdl=MODEL, metadata=METADATA, labels=curated_labels)
emb.process_data_pipeline(label=label, q=0, add_other=True)
data = emb.data
X, y = data.drop(columns=[label]).values, data[label].values

meta = emb.metadata
meta['label'] = meta['label'].apply(lambda x: re.split('(.)\[|\(|,', x)[0].strip())
test_embeddings_idx = {word:emb.mdl.wv.vocab[word].index for word in emb.mdl.wv.vocab
                       if word not in emb.train_words["word"] and "hypo.clst" in word}
X_test = emb.embedding[[*test_embeddings_idx.values()]]

trainer = NNClf(X=X, y=y, out_dir=None)
predicted, predicted_prob = trainer.model_fit(X, X_test, y)

test_df = pd.DataFrame.from_dict(test_embeddings_idx, orient="index").reset_index().rename(
    columns={0:"index", 'index':"word"})
dic_y_mapping = {n: label for n, label in enumerate(np.unique(y))}

for key, value in dic_y_mapping.items():
    test_df[value] = predicted_prob[:,key]

test_df['predicted_class'] = predicted
test_df['predicted_class_score'] = predicted_prob.max(axis=1)

test_df["weighted_total_score"] = test_df.apply(lambda row: sum([row[k]*class_2_aupr[k] for k in class_2_aupr]), axis=1)
test_df["weighted_prediction_score"] = test_df.apply(lambda row:
                                                     class_2_aupr[row["predicted_class"]] *
                                                     row["predicted_class_score"], axis=1)
test_df = test_df.sort_values(by="weighted_total_score", ascending=False)

test_df.to_pickle(os.path.join(OUTPUT_DIR,  "hypothetical_predictions.pkl"))




import logging
import os
import pandas as pd
from sklearn.model_selection import StratifiedKFold
from src.genomic_embeddings.models import MLClf, MLClfFolds, NNClf, NNClfFolds
from src.genomic_embeddings.data import Embedding
from src.genomic_embeddings.plot import ModelPlots, FoldModelPlots
import argparse


argparse = argparse.ArgumentParser()
argparse.add_argument('--model', required=True, type=str, help='model file')
argparse.add_argument('--output',
                          default='/predictions',
                          type=str, help='predictions output dir')
argparse.add_argument('--metadata',
                          default='metadata.csv',
                          type=str, help='metadata csv file path')
params = argparse.parse_args()

MODEL = params.model
METADATA = params.metadata
OUTPUT_DIR = params.output

# configure logger
logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s',
                    filename=os.path.join(OUTPUT_DIR, f"Validation.log"), level=logging.INFO)

# top predicted LABEL
top_labels = ['Amino sugar and nucleotide sugar metabolism', 'Benzoate degradation', 'Cell growth', 'Energy metabolism',
              'Methane metabolism', 'Oxidative phosphorylation', 'Prokaryotic defense system', 'Ribosome',
              'Secretion system', 'Transporters', 'Glycosyltransferases']

curated_labels = ['Amino sugar and nucleotide sugar metabolism',
                  'Benzoate degradation',
                  'Energy metabolism',
                  'Oxidative phosphorylation',
                  'Porphyrin and chlorophyll metabolism',
                  'Prokaryotic defense system',
                  'Purine metabolism',
                  'Ribosome',
                  'Secretion system',
                  'Transporters',
                  'Two-component system']

curated_labels_no_pumps = ['Amino sugar and nucleotide sugar metabolism',
                  'Benzoate degradation',
                  'Energy metabolism',
                  'Oxidative phosphorylation',
                  'Porphyrin and chlorophyll metabolism',
                  'Prokaryotic defense system',
                  'Ribosome',
                  'Secretion system',
                  'Two-component system']

labels = [(top_labels, 'TOPLABELS'), (curated_labels, 'CURATED-LABELS'),
          (curated_labels_no_pumps, 'NO-PUMPS-CURATED-LABELS')]
LABEL = 'label'
q = ''

for label, label_alias in labels:
        alias = label_alias
        logging.info(f"=== Extract embedding for label = {LABEL}, Q= {q}")
        emb = Embedding(mdl=MODEL, metadata=METADATA, labels=label)
        emb.process_data_pipeline(label=LABEL, q=q, add_other=True)
        logging.info(f"Number of effective words: {emb.effective_words.shape[0]}\n")


        data = emb.data
        X, y = data.drop(columns=[LABEL]).values, data[LABEL].values
        cv = StratifiedKFold(n_splits=5, shuffle=True, random_state=42)
        logging.info(f"Matrix shape is: {X.shape}, 20% used for testing in 5 fold CV\n"
                     f"Train size: {X.shape[0] * 0.8}, Test size: {X.shape[0] * 0.2}\n"
                     f"Number of unique classes: {pd.Series(y).nunique()}")

        MDLS = [(MLClf(X=X, y=y, out_dir=OUTPUT_DIR), alias),
                (NNClf(X=X, y=y, out_dir=OUTPUT_DIR),alias),
                (MLClfFolds(X=X, y=y, cv=cv, out_dir=OUTPUT_DIR),'FOLD_' + alias),
                (NNClfFolds(X=X, y=y, cv=cv, out_dir=OUTPUT_DIR), 'FOLD_' + alias)]


        for mdl, name in MDLS:
            mdl.classification_pipeline(LABEL, alias=name)
            if 'FOLD' in name:
                plotter = FoldModelPlots(mdl=mdl)
                plotter.plot_single_aupr_with_ci()
                plotter.plot_single_roc_with_ci()
            else:
                plotter = ModelPlots(mdl=mdl)
            plotter.plot_precision_recall()
            plotter.plot_roc()









import glob
import shutil
import os
import codecs
import argparse
import pickle

import numpy as np
from sklearn.model_selection import KFold
from tqdm import tqdm

def cp_files(list_of_files, dest):
    for f in list_of_files:
        if os.path.isfile(f):
            shutil.copy(f, dest)

def extract_known_words(list_of_files, unknown='hypo.clst'):
    corpus_raw = u""
    for f in tqdm(list_of_files):
        with codecs.open(f, "r", "utf-8") as book_file:
            corpus_raw += book_file.read()
    raw_sentences = corpus_raw.split('. ')
    words = []
    for raw_sentence in tqdm(raw_sentences):
        if len(raw_sentence) > 0:
            words.extend([w for w in raw_sentence.split() if unknown not in w])
    return set(words)

class CorpusCV():
    def __init__(self, corpus_dir, output_dir, folds, name, folds_mapping=None):
        self.corpus_dir = corpus_dir
        self.output_dir = output_dir
        self.nfolds = folds
        self.name = name
        self.folds_mapping = folds_mapping

    def create_output_dir(self):
        os.makedirs(os.path.join(self.output_dir, self.name),exist_ok=True)
        self.output_dir = os.path.join(self.output_dir, self.name)

    def corpus_kfold(self):
        corpus_files = np.array(glob.glob(self.corpus_dir))
        cv = KFold(n_splits=self.nfolds)
        fold = 1
        for train_idx, test_idx in cv.split(corpus_files):
            train_files = corpus_files[train_idx]
            test_files = corpus_files[train_idx]

            test_words = extract_known_words(test_files)

            # create the fold directory
            fold_dir = os.path.join(self.output_dir, f'fold_{fold}')
            train_dir = os.path.join(fold_dir, 'corpus')
            os.makedirs(fold_dir, exist_ok=True)
            os.makedirs(train_dir, exist_ok=True)

            cp_files(train_files, train_dir)
            np.save(os.path.join(fold_dir, 'test_words.npy'), test_words)

            fold += 1

    def corpusLOPOCV(self):
        with open(self.folds_mapping, 'rb') as handle:
            lopocv_mapper = pickle.load(handle)

        for phylum in lopocv_mapper:
            train_files = lopocv_mapper[phylum]['train_files']
            test_files = lopocv_mapper[phylum]['test_files']

            test_words = extract_known_words(test_files)

            # create the fold directory
            fold_dir = os.path.join(self.output_dir, f'fold_{phylum}')
            train_dir = os.path.join(fold_dir, 'corpus')
            os.makedirs(fold_dir, exist_ok=True)
            os.makedirs(train_dir, exist_ok=True)

            cp_files(train_files, train_dir)
            np.save(os.path.join(fold_dir, 'test_words.npy'), test_words)

if __name__ == '__main__':
    argparse = argparse.ArgumentParser()
    argparse.add_argument('--input_dir', required=True, type=str, help='input directory with the full corpus files')
    argparse.add_argument('--output_dir', required=True, type=str, help='output directory for cross validation')
    argparse.add_argument('--lopocv', default=None, type=str, help='folds mapping of corpus files')
    argparse.add_argument('--name', default='5foldcv', type=str, help='cv identifier')
    argparse.add_argument('--folds', default=5, type=int, help='number of folds in cv')

    params = argparse.parse_args()

    corpus_cv = CorpusCV(params.input_dir, params.output_dir, params.folds, params.name, params.lopocv)
    corpus_cv.create_output_dir()
    if corpus_cv.folds_mapping is None:
        corpus_cv.corpus_kfold()
    else:
        corpus_cv.corpusLOPOCV()


import pandas as pd
import numpy as np
import re

# word 2 vec
from gensim.models import word2vec as w2v


class Embedding(object):
    def __init__(self, mdl, metadata, labels=None):
        self.mdl = w2v.Word2Vec.load(mdl)
        self.metadata = pd.read_csv(metadata)
        self.labels = labels
        self.embedding = self.mdl.wv.vectors.astype('float64')
        self.known_embeddings = None
        self.word2index = None
        self.effective_words = None
        self.train_words = None
        self.data = None
        self.unknown_embeddings = None
        self.unknown_word2index = None
        self.data_with_words = None

    def extract_known_words(self, unknown="hypo.clst"):
        idxs = [self.mdl.wv.vocab[word].index for word in self.mdl.wv.vocab if unknown not in word]
        known_mat = self.embedding[idxs]
        known_word2index = {self.mdl.wv.index2word[word]: i for i, word in enumerate(idxs)}

        self.known_embeddings = known_mat
        self.word2index = known_word2index

    def extract_effective_words(self, label='label'):
        metadata = self.metadata
        metadata[label] = metadata[label].apply(lambda x: re.split('(.)\[|\(|,', x)[0].strip()) #remove redundant
        eff_words = pd.DataFrame(self.word2index.items(), columns=["word","index"])
        eff_words["KO"] = eff_words["word"].apply(lambda x: x.rsplit(".")[0])
        eff_words = eff_words.merge(metadata, on=["KO"], how='left')[["word","index",label]].dropna()
        self.effective_words = eff_words

    def filter_effective_words(self, q=0.96, label='label'):
        eff_words = self.effective_words
        labels = self.labels
        if labels is None:
            labels_count = eff_words.groupby(label).size().reset_index(name="size").\
                sort_values(by="size", ascending=False)
            labels_to_keep = labels_count[labels_count["size"] >= np.quantile(labels_count["size"], q)]
            labels_to_keep = labels_to_keep[
                ~labels_to_keep[label].isin(["Function unknown [99997]", "Enzymes with EC numbers [99980]"])]
            labels = labels_to_keep[label].values

        eff_words = eff_words[eff_words[label].isin(labels)]
        self.train_words = eff_words

        if self.labels is None:
            self.labels = eff_words[label].unique()

    def add_other_class(self, label='label', sample_size=12, min_points=30):
        eff_words = self.effective_words
        eff_words[label] = eff_words[label].apply(lambda x: re.split('(.)\[|\(|,', x)[0].strip())  # remove redundant

        label_sizes = eff_words.groupby('label').size().reset_index(name='size')
        labels_to_keep = self.labels

        sample_from = label_sizes[~label_sizes["label"].isin(labels_to_keep)].sort_values(by='size', ascending=False)
        labels_to_sample_from = sample_from[sample_from['size'] > min_points]['label']

        other_class = eff_words[eff_words['label'].isin(labels_to_sample_from)].groupby("label").sample(n=sample_size,
                                                                                             random_state=42)
        other_class['label'] = 'Other'
        data_with_other = pd.concat([self.train_words, other_class])
        self.train_words = data_with_other


    def cleanup_train_data(self, add_other=False):
        df = pd.DataFrame(self.known_embeddings)
        if add_other:
            self.add_other_class()
        df = df.reset_index().merge(self.train_words, on="index", how="right")
        self.data_with_words = df
        self.data = df.drop(columns=["index", "word"])


    def process_unknown_words(self, labels2filter, label='label'):
        meta = self.metadata
        meta['label'] = meta['label'].apply(lambda x: re.split('(.)\[|\(|,', x)[0].strip())
        if labels2filter is None:
            labels2filter = meta[label].unique()
        train_words = meta[meta[label].isin(labels2filter)]['KO'].values
        test_embeddings_idx = {word: self.mdl.wv.vocab[word].index for word in self.mdl.wv.vocab if
                               word not in train_words}
        unknown_embs = self.embedding[[*test_embeddings_idx.values()]]

        self.unknown_embeddings = unknown_embs
        self.unknown_word2index = test_embeddings_idx

    def process_data_pipeline(self, label, q, add_other=False):
        self.extract_known_words()
        self.extract_effective_words(label=label)
        self.filter_effective_words(q=q, label=label)
        self.cleanup_train_data(add_other=add_other)








from gensim.models import word2vec as w2v
import pickle
import pandas as pd


def load_embeddings(embedding_mdl):
    """
    load the existing embeddings from trained model
    :param embedding_mdl: the path to a trained w2v model
    :return: w2v object, with embeddings for each word in the corpus
    """
    return w2v.Word2Vec.load(embedding_mdl)

def get_2d_mapping(embedding_2d_path):
    """
    get the 2d coordinates as obtained by umap for each gene in the vocabulary.
    :param embedding_2d_path: a path to pickle file containing the 2d coordinates for each gene
    :return: a dataframe with a word and coordinates <x,y>
    """
    with open(embedding_2d_path, "rb") as handle:
        embedding_2d = pickle.load(handle)
    return embedding_2d

def get_functional_prediction(predicted_hypo_path):
    """
    get a table with all prediction made by the functional model
    :param predicted_hypo_path: a path to the pickle file with the hypothetical proteins
    :return: data frame with predictions for every hypothetical word
    """
    return pd.read_pickle(predicted_hypo_path, 'rb')



import os
from tqdm import tqdm
import numpy as np
import pandas as pd
import pickle
import glob
from Bio import SeqIO
import socket
import subprocess
import hashlib
from BCBio import GFF

class CorpusGenerator(object):
    def __init__(self, gff, by="annotation", include='include.txt',
                 word_mapping='word_mapping.pkl'):
        self.gff = gff
        self.name = gff.name
        self.include = include
        self.annotation = by
        self.text_df = None
        self.word_mapping = word_mapping

    def __repr__(self):
        return self.name

    def get_gff(self):
        return self.gff

    def validate(self):
        """ validate params before calling to make sentences"""
        gff = self.get_gff()
        gff.set_name()
        try:
            gff.set_gff_table()
        except:
            print("Cannot load KEGG table")
            return
        try:   
            gff.set_hypothetical()
        except:
            print("Cannot load hypothetical table")
            return
        return gff


    def make_sentences_df(self):
        gff = self.validate()
        if gff is None:
            print(f"Validation failed for GFF instance {self.gff.name}.")
            return
        hypo2rep = gff.hypothetical
        sample = gff.gff_table
        annotation = self.annotation

        with open(self.include, 'r') as o:
            includes = [l.replace('\n', '') for l in o.readlines()]

        sample["word"] = sample.apply(
            lambda row: hypo2rep[row["contig_id"]] if row["contig_id"] in hypo2rep else row[annotation],
            axis=1)
        sample['ctg'] = sample['contig_id'].apply(lambda x: x.rsplit('_', 1)[0])
        sample = sample[sample['ctg'].isin(includes)]
        sample['orf'] = sample['contig_id'].apply(lambda x: int(x.rsplit('_', 1)[-1]))

        data_to_text = sample.sort_values(["ctg", "orf"]).groupby(["ctg"])["word"].apply(
            list).reset_index()

        self.text_df = data_to_text

        return data_to_text

    def compile_text(self, path):
        """ create a text file contains the """
        text_df = self.text_df
        if text_df is None:
            print("no text df found, Exiting....")
            return

        text_df["text"] = text_df["word"].apply(lambda x: ' '.join(x))
        text = '. '.join(text_df["text"].values)
        with open(path, "w") as handle:
            handle.write(text)
        return text

    def generate(self, path):
        text_df = self.make_sentences_df()
        if text_df is None:
            return
        self.compile_text(path)











from collections import Counter
import statsmodels.stats.multitest as multi
from matplotlib.backends.backend_pdf import PdfPages
import os

import pandas as pd
import umap
import glob
import pickle
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import seaborn as sns
from scipy.stats import stats, entropy
import gensim
from gensim.models import word2vec as w2v
import re
from tqdm import tqdm

# taken from https://umap-learn.readthedocs.io/en/latest/parameters.html
def draw_umap(data, n_neighbors=15, min_dist=0.1, n_components=2, metric='euclidean', title=''):
    fit = umap.UMAP(
        n_neighbors=n_neighbors,
        min_dist=min_dist,
        n_components=n_components,
        metric=metric
    )
    u = fit.fit_transform(data)
    fig = plt.figure()
    if n_components == 1:
        ax = fig.add_subplot(111)
        ax.scatter(u[:,0], range(len(u)), c=data)
    if n_components == 2:
        ax = fig.add_subplot(111)
        ax.scatter(u[:,0], u[:,1], c=data)
    if n_components == 3:
        ax = fig.add_subplot(111, projection='3d')
        ax.scatter(u[:,0], u[:,1], u[:,2], c=data, s=100)
    plt.title(title, fontsize=18)

def reducer(matrix, mdl, n_neighbors=20, min_dist=0.1, n_components=2, metric='euclidean'):
    fit = umap.UMAP(
        n_neighbors=n_neighbors,
        min_dist=min_dist,
        n_components=n_components,
        metric=metric
    )
    u = fit.fit_transform(matrix)

    points = pd.DataFrame([tuple([word] + [coord for coord in coords])
                           for word, coords in [(word, u[mdl.wv.vocab[word].index])
                                                for word in mdl.wv.vocab]],
                          columns=["word"] + [str(i) for i in range(n_components)])
    return points

def add_metadata(metadata_path, mdl_folder):
    """
    add metadata for 2D words
    :param metadata_path: a path to a ko table with annotations per KO
    :param mdl_folder: a folder containing g2v model outputs
    :return: a merged dataframe
    """
    ko_table = pd.read_table(metadata_path)
    # read files from input folder
    cur_files = glob.glob(f"{mdl_folder}/*")
    with open([c for c in cur_files if "tsne" in c][0], 'rb') as o:
        words = pickle.load(o)
    words["hypothetical"] = words["word"].apply(lambda x: "YES" if "Cluster" in x else "NO")
    words["KO"] = words["word"]
    # merge files and save as pickle to reduce space
    merged = words.merge(ko_table, on=["KO"], how="left").fillna("unkown")
    return merged

def cluster_data(merged, cluster_obj):
    """
    cluster words data using a clustering algorithm
    :param merged: words dataframe having "x","y" columnsrr
    :param cluster_obj: abject for clustering, need to have a fit_predict function
    :return: 2d clustered dataframe
    """

    fp = getattr(cluster_obj, "fit_predict", None)
    if not callable(fp):
        raise Exception("cluster object provided do not have a fit_predict function")

    cluster_labels = cluster_obj.fit_predict(merged[["x","y"]])
    merged["cluster"] = cluster_labels
    merged = merged.sort_values(by="cluster")
    merged["cluster"] = merged["cluster"].astype(str)
    return merged


def get_kegg_enrichments(merged):
    """
    calculate all naive enrichments of kegg ko_lvl_3 annotations using a fisher exact test
    :param merged: a data frame of words, must be merged with KO data
    :return: a pair of dataframes- (merged df with enrichments, only enrichments)
    """
    res = {}
    dfs = []
    for cluster in merged["cluster"].unique():
        clust_df = merged[merged["cluster"] == cluster]
        split_by = "KO_lvl_3"
        splitted = clust_df[split_by].apply(lambda x: x.split(';')).explode()
        annotations = [s.strip() for s in splitted]
        res[cluster] = Counter(annotations)
        d = pd.DataFrame.from_dict(res[cluster], orient='index').reset_index().rename(
            columns={'index': 'annotation', 0: 'count'})
        d["cluster"] = cluster
        dfs.append(d)
    df = pd.concat(dfs)

    annot_enrich = []
    for cluster in merged["cluster"].unique():
        clust_df = df[df["cluster"] == cluster]
        not_clust_df = df[df["cluster"] != cluster]

        annotations = clust_df[clust_df["count"] > 1]["annotation"].unique()
        for annot in annotations:
            cluster_annot = clust_df[clust_df["annotation"] == annot]["count"].sum()
            not_cluster_annot = not_clust_df[not_clust_df["annotation"] == annot]["count"].sum()
            cluster_not_annot = clust_df[clust_df["annotation"] != annot]["count"].sum()
            not_cluster_not_annot = not_clust_df[not_clust_df["annotation"] != annot]["count"].sum()

            oddsratio, pvalue = stats.fisher_exact(
                [[cluster_annot, cluster_not_annot], [not_cluster_annot, not_cluster_not_annot]])
            annot_enrich.append((oddsratio, pvalue, cluster, annot))
    data = pd.DataFrame(annot_enrich, columns=["odds", "pvalue", "cluster", "annotation"])
    data['corrected_pvalue'] = multi.fdrcorrection(data['pvalue'])[1]
    data["enriched"] = data["corrected_pvalue"].apply(lambda x: "yes" if x < 0.05 else "no")

    enriched_df = data[(data["enriched"] == "yes") & (data["annotation"] != "unkown")]
    enriched_df = enriched_df.sort_values(by=["cluster", "corrected_pvalue"])
    res_df = merged.merge(data, on="cluster", how="left")

    return res_df, enriched_df


def cluster_entropy(merged, mode="collapsed"):
    """
    get the entropy of each cluster
    :param merged: a dataframe with words, kos and clusters
    :return: a dict - cluster: (score, cluster size, unknown size)
    """
    res = {}
    for cluster in merged["cluster"].unique():
        clust_df = merged[merged["cluster"] == cluster]
        split_by = "KO_lvl_3"
        if mode != "collapsed":
            splitted = clust_df[split_by].apply(lambda x: x.split(';')).explode()
            annotations = [s.strip() for s in splitted]
        else:
            annotations = clust_df[split_by]
        res[cluster] = Counter(annotations)

    cluster_scores = {}
    for cluster in res:
        vals = [v for key, v in res[cluster].items() if key!= "unkown"]
        n = sum(vals)
        score = entropy(vals, base=n)
        n_unknown = res[cluster]["unkown"]
        size = sum(res[cluster].values())
        cluster_scores[cluster] = (score, size, n_unknown)
    return cluster_scores


def process_word_statistics(mdl, outdir, hypo_word = 'hypo.clst'):
    with PdfPages(os.path.join(outdir, f'HypoDistribution.pdf')) as pdf:
        w2c_known = dict()
        w2c_unknown = dict()
        for item in mdl.wv.vocab:
            if hypo_word in item:
                w2c_unknown[item] = mdl.wv.vocab[item].count
            else:
                w2c_known[item] = mdl.wv.vocab[item].count

        fig, ax = plt.subplots(1, 2, figsize=(14, 4))

        ax[0].hist(w2c_known.values(), color='#7FB7E5', bins=20)
        ax[0].set_title(
            f"Known\nAVG: {round(np.mean(list(w2c_known.values())), 2)}, MED: {round(np.median(list(w2c_known.values())), 2)} MAX: {max(w2c_known.values())}")
        ax[0].set_yscale("log")
        ax[0].grid(True)

        ax[1].hist(w2c_unknown.values(), color='#DC3D13', alpha=0.8)
        ax[1].set_title(
            f"Unknown\nAVG: {round(np.mean(list(w2c_unknown.values())), 2)}, MED: {round(np.median(list(w2c_unknown.values())), 2)} MAX: {max(w2c_unknown.values())}")
        ax[1].set_yscale("log")
        ax[1].grid(True)

        pdf.savefig(transparent=True, bbox_inches="tight")
        plt.close()

def summeraize_mdls(w2v_mdl, word2metadata="words2metadata.pkl"):

    with open(word2metadata, 'rb') as o:
        words = pickle.load(o)

    res = []
    for mdl in tqdm(w2v_mdl):
        g2v = w2v.Word2Vec.load(mdl)
        corpus_type = "annotation" if "extended" not in mdl else "annotation_extended"
        batch_type = mdl.split("/")[6]
        mintf = int(re.findall(r"tf(\d*)_annotation", mdl)[-1])
        vocab_size = sum([v.count for k,v in g2v.wv.vocab.items()])
        unique_tokens = len(g2v.wv.vocab)
        unique_hypo = len([k for k in g2v.wv.vocab if 'hypo.clst.' in k])
        unique_kegg = unique_tokens - unique_hypo
        hypo_count = sum([v.count for k,v in g2v.wv.vocab.items() if 'hypo.clst.' in k])
        diamond_hypo = sum([words[k][0] for k in g2v.wv.vocab if 'hypo.clst.' in k and k in words])
        diamond_known_hypo = len([words[k][0] for k in g2v.wv.vocab if 'hypo.clst.' in k and k in words]) - diamond_hypo
        diamond_nf = unique_hypo - diamond_hypo - diamond_known_hypo

        res.append((corpus_type, batch_type, mintf, vocab_size, unique_tokens, unique_kegg, unique_hypo, hypo_count, diamond_hypo, diamond_known_hypo, diamond_nf))
    

    df = pd.DataFrame(res, columns=['corpus_type', 'batch_type', 'mintf', 'vocab_size', 'unique_tokens', 'unique_kegg', 'unique_hypo', 'hypo_count', 'diamond_hypo','diamond_known_hypo', 'diamond_not_found'])
    df["kegg_count"] = df["vocab_size"] - df["hypo_count"]
    df["per_kegg"] = df["kegg_count"] / df["vocab_size"]
    df["per_hypo"] = df["hypo_count"] / df["vocab_size"]
    df["per_unique_tokens"] = df["unique_tokens"] / df["vocab_size"]
    df["per_unique_kegg"] = df["unique_kegg"] / df["unique_tokens"]
    df["per_unique_hypo"] = df["unique_hypo"] / df["unique_tokens"]

    df["per_diamond_hypo"] = df["diamond_hypo"] / df["unique_hypo"]
    df["per_diamond_known_hypo"] = df["diamond_known_hypo"] / df["unique_hypo"]
    df["per_diamond_not_found"] = df["diamond_not_found"] / df["unique_hypo"]
    df = df.sort_values(by=["corpus_type", "mintf"])
    return df



# imports
import codecs
import glob
import logging
import multiprocessing
from tqdm import tqdm
import argparse
import os
import pickle
import sys
import sklearn.manifold
import pandas as pd
from datetime import datetime
import umap

#word 2 vec
from gensim.models import word2vec as w2v


class Corpus(object):
    def __init__(self, dir_path):
        self.dirs = dir_path
        self.len = None
        self.corpus = None
        self.sentences = None
        self.token_count = None

    def load_corpus(self):
        # initialize rawunicode , all text goes here
        corpus_raw = u""
        files = glob.glob(self.dirs)
        files.sort()
        print(f"Number of files in corpus: {len(files)}")
        for f in tqdm(files):
            with codecs.open(f, "r", "utf-8") as book_file:
                corpus_raw += book_file.read()

        # set current corpus
        self.corpus = corpus_raw

    def make_sentences(self, delim=". "):
        # create sentences from corpus
        if self.corpus == None:
            print("Error: no corpus object found, use load_corpus function to generate corpus object")
            return
        raw_sentences = self.corpus.split(delim)
        sentences = []
        for raw_sentence in tqdm(raw_sentences):
            if len(raw_sentence) > 0:
                sentences.append(raw_sentence.split())
        self.sentences = sentences

        # update number of tokens in corpus
        self.token_count = sum([len(sentence) for sentence in sentences])

def main(args):
    # configure logger -
    out_dir = os.path.join(args.output, args.alias)
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)
    logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s',
                        filename=os.path.join(out_dir, f"{args.alias}.log"), level=logging.INFO)

    corpus = Corpus(args.input)
    corpus.load_corpus()
    corpus.make_sentences()
    # Seed for the RNG, to make the results reproducible.
    seed = 1
    if args.workers == None:
        args.workers = multiprocessing.cpu_count()

    # build model
    gene2vec = w2v.Word2Vec(
        sg=1,
        seed=seed,
        workers=args.workers,
        size=args.size,
        min_count=args.minTF,
        window=args.window,
        sample=args.sample
    )
    gene2vec.build_vocab(corpus.sentences)
    print("Gene2Vec vocabulary length:", len(gene2vec.wv.vocab))
    gene2vec.train(corpus.sentences,
                     total_examples=gene2vec.corpus_count, epochs=args.epochs)
    # save model
    gene2vec.save(os.path.join(out_dir, f"{args.alias}_{datetime.today().strftime('%Y-%m-%d')}.w2v"))

    mapper = umap.UMAP(n_neighbors=15,min_dist=0.0, n_components=2)
    #train umap
    all_word_vectors_matrix_2d = mapper.fit_transform(gene2vec.wv.vectors.astype(
        'float64'))
    points = pd.DataFrame([(word, coords[0], coords[1])
        for word, coords in [(word, all_word_vectors_matrix_2d[gene2vec.wv.vocab[word].index])
            for word in gene2vec.wv.vocab]],
    columns=["word", "x", "y"])
    with open(os.path.join(out_dir, f"words_umap_{datetime.today().strftime('%Y-%m-%d')}"), 'wb') as o:
        pickle.dump(points, o)


if __name__ == "__main__":
    argparse = argparse.ArgumentParser()
    argparse.add_argument('--window', default=5, type=int, help='window size')
    argparse.add_argument('--size', default=300, type=int, help='vector size')
    argparse.add_argument('--workers', required=False, type=int, help='number of processes')
    argparse.add_argument('--epochs', default=5, type=int, help='number of epochs')
    argparse.add_argument('--minTF', default=4, type=int, help='minimum term frequency')
    argparse.add_argument('--sample', default=1e-3, type=int, help='down sampling setting for frequent words')
    argparse.add_argument('--model', required=False, type=str, help='model file if exists')
    argparse.add_argument('--input', default='../data/*', type=str, help='dir to learn from, as a regex for file generation')
    argparse.add_argument('--output', default='outputs/', type=str, help='output folder for results')
    argparse.add_argument('--alias', default='G2V', type=str, help='model running alias that will be used for model tracking')
    params = argparse.parse_args()

    main(params)



import pickle
import xgboost as xgb
from sklearn.model_selection import  StratifiedKFold
from sklearn.ensemble import RandomForestClassifier
from sklearn.svm import SVC
from sklearn.model_selection import GridSearchCV
import pandas as pd
import os

import argparse


def optimize(clf, X, y, parameters, alias):
    cv=StratifiedKFold(n_splits=5, shuffle=True, random_state=42)
    scores = ["precision", "recall"]

    results = []

    for score in scores:
        print("# Tuning hyper-parameters for %s" % score)
        print()

        mdl = GridSearchCV(estimator=clf, param_grid=parameters, cv=cv, scoring="%s_weighted" % score)
        mdl.fit(X, y)

        res = pd.DataFrame(mdl.cv_results_)
        res['score_function'] = score
        results.append(res)

    return pd.cocnat(results)



def main(args):

    with open(args.fold2data, 'rb') as o:
        fold2data = pickle.load(o)

    name = args.clf_name
    fold = args.fold_name
    lopocv = fold2data[args.cv_name]

    X_train, y_train = lopocv[fold]['X_train'], lopocv[fold]['y_train']

    names_mappings = { 'SVC' : {'CLF':SVC(),
                                'params': {'kernel':('linear', 'rbf'), 'C':[0.01, 1, 10]}},
                       'RF': {'CLF': RandomForestClassifier(n_jobs=10),
                              'params': {'max_depth': [6, 10, 50, 100, None], 'max_features': ['auto', 'sqrt'],
                                         'min_samples_leaf': [1, 2, 4], 'min_samples_split': [2, 5, 10],
                                         'n_estimators': [100, 500, 1000]}},
                       'XGB' : {'CLF':xgb.XGBClassifier(n_jobs=10),
                                'params': {'n_estimators': [100, 500, 800, 1000], 'max_depth': [6, 10, 50, 100, None],
                                           'learning_rate': [0.001, 0.05, None]}}
    }

    clf, param_grid  = names_mappings[name]['CLF'], names_mappings[name]['params']
    results = optimize(clf, X_train, y_train, param_grid, name)
    results['fold_name'] = fold
    results['model'] = name

    results.to_csv(os.path.join(args.outdir, f'opt_{name}_{fold}'), index=False)


if __name__ == '__main__':
    argparse = argparse.ArgumentParser()
    argparse.add_argument('--fold_name', required=True, type=str, help='name of the fold use for train')
    argparse.add_argument('--cv_name', default='LOPOCV', type=str, help='name of the CV use for train')
    argparse.add_argument('--clf_name', default='SVC', type=str, help='name of the CV use for train')
    argparse.add_argument('--outdir', default='./',
                          type=str, help='output dir to save files')
    argparse.add_argument('--fold2data',
                          default='fold2data.pkl',
                          type=str, help='mapping of LOPOCV train and test')

    args = argparse.parse_args()

    main(args)


import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
from itertools import cycle
import math
import os
from sklearn import metrics
import numpy as np
import pickle

import matplotlib
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

class ModelPlots(object):
    def __init__(self, mdl):
        self.mdl = mdl
        self.roc_data = mdl.roc
        self.precision_data = mdl.pr
        self.mdl_report = mdl.report
        self.out_dir = mdl.out_dir
        self.name = mdl.name

    def plot_roc(self):
        roc_df = self.roc_data
        res = self.mdl_report

        colors = ['pink', 'turquoise', 'darkorange', 'cornflowerblue', 'teal', 'gold', 'olive','tomato', 'deeppink']

        with PdfPages(os.path.join(self.out_dir, f'{self.name}_ROC.pdf')) as pdf:
            fig, ax = plt.subplots(figsize=(5, 4))

            for c, cl in zip(cycle(colors), roc_df["class"].unique()):
                if cl == "ALL":
                    c = "k"
                    score = metrics.auc(roc_df[roc_df["class"] == cl]["fpr"], roc_df[roc_df["class"] == cl]["tpr"])
                else:
                    score = res.groupby(res.index)['auc'].mean().loc[cl]

                ax.plot(roc_df[roc_df["class"] == cl]["fpr"], roc_df[roc_df["class"] == cl]["tpr"], lw=3,
                        label="class {0} ({1:0.2f})".format(cl, score),
                        color=c)
            ax.plot([0, 1], [0, 1], color='grey', lw=3, linestyle='--', alpha=0.2)
            ax.set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"ROC",
                   xlabel="False Positive Rate", ylabel="True Positive rate")
            ax.grid(True)
            ax.text(.7, 0.05, f"AUC:{self.mdl.auc}", fontsize=12)
            plt.legend(bbox_to_anchor=(1.01, 1))
            pdf.savefig(transparent=True, bbox_inches="tight")
            plt.close()

    def plot_precision_recall(self):
        pr_df = self.precision_data
        res = self.mdl_report

        colors = ['pink', 'turquoise', 'darkorange', 'cornflowerblue', 'teal', 'gold', 'olive','tomato', 'deeppink']
        with PdfPages(os.path.join(self.out_dir, f'{self.name}_AUPR.pdf')) as pdf:
            fig, ax = plt.subplots(figsize=(5, 4))
            for c, cl in zip(cycle(colors), pr_df["class"].unique()):

                if cl == "ALL":
                    c = "k"
                    score = metrics.auc(pr_df[pr_df["class"] == cl]["recall"], pr_df[pr_df["class"] == cl]["precision"])
                else:
                    score = res.groupby(res.index)['aupr'].mean().loc[cl]

                ax.plot(pr_df[pr_df["class"] == cl]["recall"], pr_df[pr_df["class"] == cl]["precision"], lw=3,
                        label="class {0} ({1:0.2f})".format(cl, score),
                        color=c)
            ax.set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"AUPR", xlabel="Recall", ylabel="Precision")
            ax.grid(True)
            ax.text(0.01, 0.05, "F1:{0:0.2f}".format(res.groupby(res.index)['f1-score'].mean().loc['weighted avg']),
                    fontsize=12)
            plt.legend(bbox_to_anchor=(1.01, 1))
            pdf.savefig(transparent=True, bbox_inches="tight")
            plt.close()


class FoldModelPlots(object):
    def __init__(self, mdl):
        self.mdl = mdl
        self.roc_data = mdl.roc
        self.precision_data = mdl.pr
        self.mdl_report = mdl.report
        self.outdir = mdl.out_dir
        self.name = mdl.name

    def plot_roc(self):
        roc_df = self.roc_data
        res = self.mdl_report

        roc_df = roc_df[roc_df['class'] != 'ALL']

        n_classes = roc_df["class"].nunique()
        classes = list(roc_df["class"].unique())
        COLS = 4
        ROWS = math.ceil(n_classes / COLS)
        pages = math.ceil(ROWS / 4)

        colors = ['pink', 'turquoise', 'darkorange', 'cornflowerblue', 'teal', 'gold', 'olive','tomato', 'deeppink']

        with PdfPages(os.path.join(self.outdir, f'{self.name}_ROC.pdf')) as pdf:
            for page in range(pages):
                fig, ax = plt.subplots(COLS, COLS, figsize=(20,16))

                i, j = 0, 0
                for cl in classes[page*COLS*COLS: page*COLS*COLS +COLS*COLS]:
                    d = roc_df[roc_df["class"] == cl]

                    for c, fold in zip(colors, d["fold"].unique()):
                        ax[i][j].plot(d[d["fold"] == fold]["fpr"], d[d["fold"] == fold]["tpr"],
                                      lw=3, label=f"fold: {fold}", color=c)
                        ax[i][j].plot([0, 1], [0, 1], color='grey', lw=3, linestyle='--', alpha=0.2)
                        ax[i][j].set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"{cl}")
                        ax[i][j].grid(True)
                        ax[i][j].text(.6, 0.05, "AUC:{0:0.2f}".format(res.groupby(res.index)['auc'].mean().loc[cl]),
                                      fontsize=12)
                        if j == 0:
                            ax[i][j].set(ylabel="True Positive Rate")
                        if i == COLS - 1:
                            ax[i][j].set(xlabel="False Positive Rate")
                    if j == (COLS-1):
                        j = 0
                        i += 1
                    else:
                        j += 1
                plt.subplots_adjust(hspace=.3, wspace=.3)
                _ = [ax[k][q].axis("off") for k in range(COLS) for q in range(COLS) if not ax[k][q].lines]
                pdf.savefig(transparent=True, bbox_inches="tight")
                plt.close()

    def plot_roc_by_fold(self):
        roc_df = self.roc_data
        res = self.mdl_report

        roc_df = roc_df[roc_df['class'] != 'ALL']
        roc_df['fold'] = roc_df['fold'].apply(lambda x: x.split('_')[-1])

        n_folds = roc_df["fold"].nunique()
        folds = list(roc_df["fold"].unique())
        COLS = 3
        ROWS = math.ceil(n_folds / COLS)
        pages = math.ceil(ROWS / 3)

        class2color = {'Amino sugar and nucleotide sugar metabolism':'DarkOrchid',
                       'Benzoate degradation':'darkorange', 'Energy metabolism':'cornflowerblue', 'Other':'grey',
                       'Oxidative phosphorylation':'gold',
                       'Porphyrin and chlorophyll metabolism':'teal',
                       'Prokaryotic defense system':'tomato', 'Ribosome':'deeppink', 'Secretion system':'pink',
                       'Two-component system':'turquoise', 'ALL':'k'}

        with PdfPages(os.path.join(self.outdir, f'{self.name}_ROC_BY_FOLD.pdf')) as pdf:
            for page in range(pages):
                fig, ax = plt.subplots(COLS, COLS, figsize=(20,16))

                i, j = 0, 0
                for fold in folds[page*COLS*COLS: page*COLS*COLS +COLS*COLS]:
                    d = roc_df[roc_df["fold"] == fold]

                    for cl in d["class"].unique():
                        ax[i][j].plot(d[d["class"] == cl]["fpr"], d[d["class"] == cl]["tpr"],
                                      lw=3, label=cl, color=class2color[cl])
                        ax[i][j].plot([0, 1], [0, 1], color='grey', lw=3, linestyle='--', alpha=0.2)
                        ax[i][j].set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"{fold}")
                        ax[i][j].grid(True)
                        ax[i][j].text(.6, 0.05, "AUC:{0:0.2f}".format(res.groupby('fold')['auc'].mean().loc[f'fold_{fold}']),
                                      fontsize=12)
                        if j == 0:
                            ax[i][j].set(ylabel="True Positive Rate")
                        if i == COLS - 1:
                            ax[i][j].set(xlabel="False Positive Rate")
                    if j == (COLS-1):
                        j = 0
                        i += 1
                    else:
                        j += 1
                plt.subplots_adjust(hspace=.3, wspace=.3)
                _ = [ax[k][q].axis("off") for k in range(COLS) for q in range(COLS) if not ax[k][q].lines]
                pdf.savefig(transparent=True, bbox_inches="tight")
                plt.close()


    def plot_precision_recall(self):
        pr_df = self.precision_data
        res = self.mdl_report

        pr_df = pr_df[pr_df['class'] != 'ALL']

        n_classes = pr_df["class"].nunique()
        classes = list(pr_df["class"].unique())
        COLS = 4
        ROWS = math.ceil(n_classes / COLS)
        pages = math.ceil(ROWS / 4)

        colors = ['pink', 'turquoise', 'darkorange', 'cornflowerblue', 'teal', 'gold', 'olive','tomato', 'deeppink']

        with PdfPages(os.path.join(self.outdir, f'{self.name}_AUPR.pdf')) as pdf:
            for page in range(pages):
                fig, ax = plt.subplots(COLS, COLS, figsize=(20,16))

                i, j = 0, 0
                for cl in classes[page*COLS*COLS : COLS*COLS*(page +1)]:

                    d = pr_df[pr_df["class"] == cl]

                    for c, fold in zip(colors, d["fold"].unique()):
                        ax[i][j].plot(d[d["fold"] == fold]["recall"], d[d["fold"] == fold]["precision"], lw=3,
                                      label=f"fold: {fold}", color=c
                                      )
                        ax[i][j].set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"{cl}")
                        ax[i][j].grid(True)
                        ax[i][j].text(0.01, 0.05, "AUPR:{0:0.2f}".format(res.groupby(res.index)['aupr'].mean().loc[cl]),
                                      fontsize=12)
                        if j == 0:
                            ax[i][j].set(ylabel="Precision")
                        if i == COLS - 1:
                            ax[i][j].set(xlabel="Recall")

                    if j == (COLS-1):
                        j = 0
                        i += 1
                    else:
                        j += 1
                plt.subplots_adjust(hspace=.3, wspace=.3)
                _ = [ax[k][q].axis("off") for k in range(COLS) for q in range(COLS) if not ax[k][q].lines]
                pdf.savefig(transparent=True, bbox_inches="tight")
                plt.close()


    def plot_precision_recall_by_fold(self):
        pr_df = self.precision_data
        res = self.mdl_report

        pr_df = pr_df[pr_df['class'] != 'ALL']
        pr_df['fold'] = pr_df['fold'].apply(lambda x: x.split('_')[-1])

        n_folds = pr_df["fold"].nunique()
        folds = list(pr_df["fold"].unique())
        COLS = 3
        ROWS = math.ceil(n_folds / COLS)
        pages = math.ceil(ROWS / 3)

        class2color = {'Amino sugar and nucleotide sugar metabolism':'DarkOrchid',
                       'Benzoate degradation':'darkorange', 'Energy metabolism':'cornflowerblue', 'Other':'grey',
                       'Oxidative phosphorylation':'gold',
                       'Porphyrin and chlorophyll metabolism':'teal',
                       'Prokaryotic defense system':'tomato', 'Ribosome':'deeppink', 'Secretion system':'pink',
                       'Two-component system':'turquoise', 'ALL':'k'}


        with PdfPages(os.path.join(self.outdir, f'{self.name}_AUPR_BY_FOLD.pdf')) as pdf:
            for page in range(pages):
                fig, ax = plt.subplots(COLS, COLS, figsize=(20,16))

                i, j = 0, 0
                for fold in folds[page*COLS*COLS : COLS*COLS*(page +1)]:

                    d = pr_df[pr_df["fold"] == fold]
                    d = d[~((d['precision'] == 0) & (d['recall'] == 0))]

                    for cl in d["class"].unique():
                        ax[i][j].plot(d[d["class"] == cl]["recall"], d[d["class"] == cl]["precision"], lw=3,
                                      label=cl, color=class2color[cl]
                                      )
                        ax[i][j].set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"{fold}")
                        ax[i][j].grid(True)
                        ax[i][j].text(0.01, 0.05, "AUPR:{0:0.2f}".format(res.groupby('fold')['aupr'].mean().loc[f'fold_{fold}']),
                                      fontsize=12)
                        if j == 0:
                            ax[i][j].set(ylabel="Precision")
                        if i == COLS - 1:
                            ax[i][j].set(xlabel="Recall")

                    if j == (COLS-1):
                        j = 0
                        i += 1
                    else:
                        j += 1
                plt.subplots_adjust(hspace=.3, wspace=.3)
                _ = [ax[k][q].axis("off") for k in range(COLS) for q in range(COLS) if not ax[k][q].lines]
                pdf.savefig(transparent=True, bbox_inches="tight")
                plt.close()


    def plot_single_aupr_with_ci(self):
        data = self.mdl.folds_pr
        overall_data = self.mdl.mean_pr
        overall_data = overall_data[~((overall_data['precision'] == 0) & (overall_data['recall'] == 0))]
        class2aupr = {}
        class2color = {'Amino sugar and nucleotide sugar metabolism':'DarkOrchid',
                       'Benzoate degradation':'darkorange', 'Energy metabolism':'cornflowerblue', 'Other':'grey',
                       'Oxidative phosphorylation':'gold',
                       'Porphyrin and chlorophyll metabolism':'teal',
                       'Prokaryotic defense system':'tomato', 'Ribosome':'deeppink', 'Secretion system':'pink',
                       'Two-component system':'turquoise', 'ALL':'k'}


        with PdfPages(os.path.join(self.outdir, f'{self.name}_AUPR_CI.pdf')) as pdf:
            fig, ax = plt.subplots(figsize=(5, 4))
            for cl in data["class"].unique():

                class_data = data[data["class"] == cl]
                tprs = class_data['interp'].tolist()[0]
                mean_precision = np.mean(tprs, axis=0)
                mean_recall = np.linspace(0, 1, mean_precision.shape[0])
                mean_auc = metrics.auc(mean_recall, mean_precision)

                if cl == 'ALL':
                    ax.plot(mean_recall, mean_precision, color=class2color[cl], label="{0} ({1:0.2f})".format(cl, mean_auc), lw=3, alpha=.8)
                else:
                    pr_data = overall_data[overall_data['class'] == cl]
                    mean_auc = metrics.auc(pr_data['recall'], pr_data['precision'])
                    ax.plot(pr_data['recall'], pr_data['precision'], color=class2color[cl],label="{0} ({1:0.2f})".format(cl, mean_auc), lw=3, alpha=.8)

                    mean_precision = np.mean(tprs)
                    std_precision = np.std(tprs, axis=0)
                    ax.fill_between(pr_data['recall'], pr_data['precision'] + std_precision, pr_data['precision'] - std_precision, color=class2color[cl], alpha=.1)

                class2aupr[cl] = mean_auc
            ax.set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"AUPR", xlabel="Recall", ylabel="Precision")
            ax.grid(True)
            plt.legend(bbox_to_anchor=(1.01, 1))
            pdf.savefig(transparent=True, bbox_inches="tight")
            plt.close()
        with open(os.path.join(self.outdir, f'{self.name}_AUPR_CI.pkl'), 'wb') as o:
            pickle.dump(class2aupr, o)

    def plot_single_roc_with_ci(self):
        data = self.mdl.folds_roc

        class2color = {'Amino sugar and nucleotide sugar metabolism':'DarkOrchid',
                       'Benzoate degradation':'darkorange', 'Energy metabolism':'cornflowerblue', 'Other':'grey',
                       'Oxidative phosphorylation':'gold',
                       'Porphyrin and chlorophyll metabolism':'teal',
                       'Prokaryotic defense system':'tomato', 'Ribosome':'deeppink', 'Secretion system':'pink',
                       'Two-component system':'turquoise', 'ALL':'k'}

        with PdfPages(os.path.join(self.outdir, f'{self.name}_ROC_CI.pdf')) as pdf:
            fig, ax = plt.subplots(figsize=(5, 4))
            for cl in data["class"].unique():

                class_data = data[data["class"] == cl]
                tprs = class_data['interp'].tolist()[0]
                mean_tpr = np.mean(tprs, axis=0)
                mean_fpr = np.linspace(0, 1, 100)
                mean_auc = metrics.auc(mean_fpr, mean_tpr)

                ax.plot(mean_fpr, mean_tpr, color=class2color[cl],
                        label="{0} ({1:0.2f})".format(cl, mean_auc), lw=3, alpha=.8)

                std_tpr = np.std(tprs, axis=0)
                tprs_upper = np.minimum(mean_tpr + std_tpr, 1)
                tprs_lower = np.maximum(mean_tpr - std_tpr, 0)
                ax.fill_between(mean_fpr, tprs_lower, tprs_upper, color=class2color[cl], alpha=.1)

            ax.set(xlim=[-0.05, 1.0], ylim=[0.0, 1.05], title=f"ROC", xlabel="True Positive Rate",
                   ylabel="False Positive Rate")
            ax.grid(True)
            plt.legend(bbox_to_anchor=(1.01, 1))
            pdf.savefig(transparent=True, bbox_inches="tight")
            plt.close()


from src.genomic_embeddings import Gff, corpus
import os
import argparse


def main(args):
    gff_file = args.input
    hypothetical = args.hypothetical

    gff = Gff.Gff(gff_path=gff_file)
    gff.set_name()
    gff.parse_gff()
    gff.extract_hypothetical_and_prokka()
    
    if args.build_corpus:
        gff = Gff.Gff(gff_path=gff_file, hypothetical_folder=hypothetical)
        gff.set_name()
        gff_corpus = corpus.CorpusGenerator(gff=gff, by=args.annotation)
        gff_corpus.generate(os.path.join(args.output, f"{gff_corpus.gff.name}.txt"))



if __name__ == "__main__":
    argparse = argparse.ArgumentParser()
    argparse.add_argument('--input', required=True, type=str, help='gff input file')
    argparse.add_argument('--output', default='/output/',
                          type=str, help='the path to restore output txt files')
    argparse.add_argument('--hypothetical',
                          default='pkl_by_sample',
                          type=str, help='hypothetical pkl per sample folder')
    argparse.add_argument('--alias', default='G2V', type=str, help='model running alias that will be used for model tracking')
    argparse.add_argument('--cluster', dest='cluster', action='store_true', help="run on cluster flag, default")
    argparse.add_argument('--local', dest='cluster', action='store_false', help="run locally flag")
    argparse.add_argument('--annotation', default='annotation', type=str, help='annotation level, can be annotation or annotation_extended [default: annotation]')
    argparse.add_argument('--build_corpus', dest='build_corpus', action='store_true', help="build corpus from parsed GFFs flag")
    argparse.set_defaults(cluster=True, build_corpus=False)
    params = argparse.parse_args()

    main(params)





import pandas as pd
import numpy as np
import os
import pickle

# ML packages
from sklearn import metrics
from sklearn import model_selection
import xgboost as xgb
from sklearn.model_selection import  StratifiedKFold
from sklearn.ensemble import RandomForestClassifier
from sklearn.svm import SVC
# DL packages
import tensorflow as tf


##### Models interface ######

class Model(object):
    def __init__(self, X, y, out_dir, clf=None):
        self.X = X
        self.y = y
        self.clf = clf
        self.out_dir = out_dir
        self.name = "MDL"
        self.report = None
        self.confusion_matrix = None
        self.pr = None
        self.roc = None
        self.auc = None
        self.ap = None
        self.history=None


    def set_alias(self, alias='_TOPLABELS'):
        self.name = self.name + alias

    def model_fit(self, X_train, X_test, y_train):
        clf = self.clf
        clf.fit(X_train, y_train)
        predicted = clf.predict(X_test)
        predicted_prob = clf.predict_proba(X_test)
        return predicted, predicted_prob


    def summarize_accuracy(self, y_test, predicted, predicted_prob, fold):
        report = metrics.classification_report(y_test, predicted, output_dict=True)
        classes = np.unique(y_test)
        y_test_array = pd.get_dummies(y_test, drop_first=False).values

        pr_dfs = []
        roc_dfs = []
        for i in range(len(classes)):
            precision, recall, thresholds = metrics.precision_recall_curve(
                y_test_array[:, i], predicted_prob[:, i])
            report[classes[i]]["aupr"] = metrics.auc(recall, precision)

            fpr, tpr, thresholds = metrics.roc_curve(y_test_array[:, i],
                                                     predicted_prob[:, i])
            report[classes[i]]["auc"] = metrics.auc(fpr, tpr)
            cur_df_pr = pd.DataFrame({"precision": precision, "recall": recall, "fold": fold, "class": classes[i]})
            cur_df_roc = pd.DataFrame({"fpr": fpr, "tpr": tpr, "fold": fold, "class": classes[i]})
            pr_dfs.append(cur_df_pr)
            roc_dfs.append(cur_df_roc)
        try:
            precision, recall, _ = metrics.precision_recall_curve(y_test_array.ravel(),
                                                                  predicted_prob.ravel())
            fpr, tpr, thresholds = metrics.roc_curve(y_test_array.ravel(), predicted_prob.ravel())
            micro_pr = pd.DataFrame({"precision": precision, "recall": recall, "fold": "micro", "class": "ALL"})
            micro_roc = pd.DataFrame({"fpr": fpr, "tpr": tpr, "fold": "micro", "class": "ALL"})
            pr_dfs.append(micro_pr)
            roc_dfs.append(micro_roc)
            auc = round(metrics.roc_auc_score(y_test, predicted_prob,
                                              multi_class="ovr", average='weighted'), 2)
            ap = round(metrics.average_precision_score(y_test_array, predicted_prob,average="micro"), 2)
            self.auc = auc
            self.ap = ap
        except:
            print("Cannot calculate accuracy metrics")

        report_df = pd.DataFrame(report).T
        report_df["fold"] = fold

        pr = pd.concat(pr_dfs)
        roc = pd.concat(roc_dfs)

        return report_df, pr, roc


    def split_and_classify(self):
        X = self.X
        y = self.y
        X_train, X_test, y_train, y_test = model_selection.train_test_split(X, y, test_size=0.2, random_state=42,
                                                                            stratify=y)
        predicted, predicted_prob = self.model_fit(X_train, X_test, y_train)
        report, precision_report, roc_report = self.summarize_accuracy(y_test, predicted, predicted_prob, fold='NO-CV')

        cm = metrics.confusion_matrix(y_test, predicted)
        classes = np.unique(y_test)
        cm = pd.DataFrame(cm, columns=classes, index=classes)

        self.pr = precision_report
        self.roc = roc_report
        self.report = report
        self.confusion_matrix = cm

        return report, precision_report, roc_report

    def wrap_up(self, label, alias):
        label_dir = os.path.join(self.out_dir, label)
        q_dir = os.path.join(label_dir, alias)
        if not os.path.exists(label_dir):
            os.makedirs(label_dir)
        if not os.path.exists(q_dir):
            os.makedirs(q_dir)

        report_path = os.path.join(q_dir, f"{self.name}_report.csv")
        cm_path = os.path.join(q_dir, f"{self.name}_confusion_matrix.csv")
        history_path = os.path.join(q_dir, f"{self.name}_history.pickle")

        self.out_dir = q_dir
        report = self.report.reset_index().rename(columns={"index": label})
        report.to_csv(report_path, index=False)
        cm = self.confusion_matrix
        history = self.history
        if cm is not None:
            cm.to_csv(cm_path)
        if history is not None:
            with open(history_path, 'wb') as handle:
                pickle.dump(history.history, handle)

    def classification_pipeline(self, label, alias='_TOPLABELS'):
        self.set_alias(alias)
        self.split_and_classify()
        self.wrap_up(label, alias)

class FoldsModel(Model):
    def __init__(self, cv=StratifiedKFold(n_splits=5, shuffle=True, random_state=42), **kwargs):
        super().__init__(**kwargs)
        self.name = self.name + 'Folds'
        self.cv = cv
        self.folds_pr = None
        self.folds_roc = None
        self.mean_pr = None

    def calc_ovelall_pr_by_folds(self, y_test_list, predicted_prob_list):
        res = []
        for fold in range(len(y_test_list)):
            classes = np.unique(y_test_list[fold])
            y_test_array = pd.get_dummies(y_test_list[fold], drop_first=False).values
            predicted_prob = predicted_prob_list[fold]

            for i in range(len(classes)):
                fold_by_class_pr = pd.DataFrame({'class':classes[i],'y_test':y_test_array[:, i],
                                                 'predicted_prob':predicted_prob[:, i]})
                res.append(fold_by_class_pr)
        df = pd.concat(res)

        all_classes = []
        for cl in df['class'].unique():
            cl_df = df[df['class'] == cl]
            precision, recall, thresh = metrics.precision_recall_curve(cl_df['y_test'], cl_df['predicted_prob'])
            res = pd.DataFrame({'class':cl, 'precision':precision, 'recall':recall, 'thresh': np.insert(thresh, 0,0)})
            res = res[~((res['precision'] == 0) & (res['recall'] == 0))].sort_values(by='thresh')
            all_classes.append(res)
        self.mean_pr = pd.concat(all_classes)

    def split_and_classify(self):
        reports, prs, rocs = [], [], []
        fold = 1
        X = self.X
        y = self.y

        y_real = []
        y_proba = []

        for train_index, test_index in self.cv.split(X, y):
            X_train, X_test = X[train_index], X[test_index]
            y_train, y_test = y[train_index], y[test_index]

            predicted, predicted_prob = self.model_fit(X_train, X_test, y_train)
            fold_report, fold_pr, fold_roc = self.summarize_accuracy(y_test, predicted, predicted_prob, fold)
            reports.append(fold_report)
            prs.append(fold_pr)
            rocs.append(fold_roc)
            y_real.append(y_test)
            y_proba.append(predicted_prob)
            fold += 1

        self.pr = pd.concat(prs)
        self.roc = pd.concat(rocs)
        self.report = pd.concat(reports)

        self.calc_ovelall_pr_by_folds(y_real, y_proba)

        return self.report, self.pr, self.roc

    def merge_folds(self):
        pr = self.pr
        roc = self.roc
        overall_by_cl = self.mean_pr

        mean_precision = np.linspace(0, 1, 100)
        mean_fpr = np.linspace(0, 1, 100)

        pr_grp = pr.groupby(["class", "fold"]).agg({'precision': list, 'recall': list}).reset_index()
        pr_grp["interp"] = pr_grp.apply(lambda row: np.interp(np.linspace(0,1,max(100, overall_by_cl[overall_by_cl['class'] == row['class']].shape[0])), row['precision'], row['recall']), axis=1)
        pr_grp["auc"] = pr_grp.apply(lambda row: metrics.auc(sorted(row["precision"]),
                                                             sorted(row["recall"], reverse=True)), axis=1)
        pr_data = pr_grp.groupby("class").agg({"interp": list, "auc": list}).reset_index()

        roc_grp = roc.groupby(["class", "fold"]).agg({'fpr': list, 'tpr': list}).reset_index()
        roc_grp["interp"] = roc_grp.apply(lambda row: np.interp(mean_fpr, row['fpr'], row['tpr']), axis=1)
        roc_grp["auc"] = roc_grp.apply(lambda row: metrics.auc(sorted(row["fpr"]),
                                                               sorted(row["tpr"], reverse=True)), axis=1)
        roc_data = roc_grp.groupby("class").agg({"interp": list, "auc": list}).reset_index()

        self.folds_roc = roc_data
        self.folds_pr = pr_data


    def merge_folds_reports(self):
        res = self.report
        res = res[res['fold'] != 'micro']
        avg = res.drop(columns=['fold']).groupby(res.index).mean()
        avg["fold"] = 'AVG'
        res = pd.concat([res, avg], axis=0)
        self.report = res
        return res

    def classification_pipeline(self, label, alias='_TOPLABELS'):
        self.set_alias(alias)
        self.split_and_classify()
        self.merge_folds_reports()
        self.merge_folds()
        self.wrap_up(label, alias)

class CVFoldsModel(FoldsModel):
    def __init__(self, fold2data, fold_type, **kwargs):
        super().__init__(**kwargs)
        self.name = self.name + fold_type + 'Folds'
        self.fold2data = fold2data

    def split_and_classify(self):
        reports, prs, rocs = [], [], []
        fold2data = self.fold2data

        y_real = []
        y_proba = []

        for fold in fold2data:
            X_train, X_test = fold2data[fold]['X_train'], fold2data[fold]['X_test']
            y_train, y_test = fold2data[fold]['y_train'], fold2data[fold]['y_test']

            predicted, predicted_prob = self.model_fit(X_train, X_test, y_train)
            fold_report, fold_pr, fold_roc = self.summarize_accuracy(y_test, predicted, predicted_prob, fold)
            reports.append(fold_report)
            prs.append(fold_pr)
            rocs.append(fold_roc)
            y_real.append(y_test)
            y_proba.append(predicted_prob)

        self.pr = pd.concat(prs)
        self.roc = pd.concat(rocs)
        self.report = pd.concat(reports)

        self.calc_ovelall_pr_by_folds(y_real, y_proba)

        return self.report, self.pr, self.roc



##### Models ######
class MLClf(Model):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.clf = xgb.XGBClassifier(n_estimators=100, max_depth=5)
        self.name = "XGB"

class NNClf(Model):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name = "DNN"


    def set_clf(self, n):
        model = tf.keras.models.Sequential([tf.keras.layers.Dense(256, activation=tf.nn.relu),
                                            tf.keras.layers.Dropout(0.2),
                                            tf.keras.layers.Dense(128, activation=tf.nn.relu),
                                            tf.keras.layers.Dropout(0.2),
                                            tf.keras.layers.Dense(64, activation=tf.nn.relu),
                                            tf.keras.layers.Dense(n, activation=tf.nn.softmax)])
        model.compile(loss='sparse_categorical_crossentropy', optimizer='adam', metrics=['accuracy'])
        self.clf = model

    def model_fit(self, X_train, X_test, y_train):
        self.set_clf(pd.Series(y_train).nunique())
        clf = self.clf
        dic_y_mapping = {n: label for n, label in
                         enumerate(np.unique(y_train))}
        inverse_dic = {v: k for k, v in dic_y_mapping.items()}
        y_train_tag = np.array([inverse_dic[y] for y in y_train])

        history = clf.fit(x=X_train, y=y_train_tag, batch_size=256,
                          epochs=20, shuffle=True, verbose=0)
        self.history = history

        predicted_prob = self.clf.predict(X_test, workers=5)
        predicted = [dic_y_mapping[np.argmax(pred)] for pred in
                     predicted_prob]
        return predicted, predicted_prob


class MLClfFolds(FoldsModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.clf = xgb.XGBClassifier(n_estimators=100, max_depth=5)
        self.name = "XGBFolds"


class NNClfFolds(FoldsModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name = "DNNFolds"

    def set_clf(self, n):
        model = tf.keras.models.Sequential([tf.keras.layers.Dense(256, activation=tf.nn.relu),
                                            tf.keras.layers.Dropout(0.2),
                                            tf.keras.layers.Dense(128, activation=tf.nn.relu),
                                            tf.keras.layers.Dropout(0.2),
                                            tf.keras.layers.Dense(64, activation=tf.nn.relu),
                                            tf.keras.layers.Dense(n, activation=tf.nn.softmax)])
        model.compile(loss='sparse_categorical_crossentropy', optimizer='adam', metrics=['accuracy'])
        self.clf = model

    def model_fit(self, X_train, X_test, y_train):
        self.set_clf(pd.Series(y_train).nunique())
        clf = self.clf
        dic_y_mapping = {n: label for n, label in
                         enumerate(np.unique(y_train))}
        inverse_dic = {v: k for k, v in dic_y_mapping.items()}
        y_train_tag = np.array([inverse_dic[y] for y in y_train])

        history = clf.fit(x=X_train, y=y_train_tag, batch_size=256,
                          epochs=20, shuffle=True, verbose=0)

        self.history = history

        predicted_prob = self.clf.predict(X_test)
        predicted = [dic_y_mapping[np.argmax(pred)] for pred in
                     predicted_prob]
        return predicted, predicted_prob

class NNClfCVFolds(CVFoldsModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name = "DNN" + self.name

    def set_clf(self, n):
        model = tf.keras.models.Sequential([tf.keras.layers.Dense(256, activation=tf.nn.relu),
                                            tf.keras.layers.Dropout(0.2),
                                            tf.keras.layers.Dense(128, activation=tf.nn.relu),
                                            tf.keras.layers.Dropout(0.2),
                                            tf.keras.layers.Dense(64, activation=tf.nn.relu),
                                            tf.keras.layers.Dense(n, activation=tf.nn.softmax)])
        model.compile(loss='sparse_categorical_crossentropy', optimizer='adam', metrics=['accuracy'])
        self.clf = model

    def model_fit(self, X_train, X_test, y_train):
        self.set_clf(pd.Series(y_train).nunique())
        clf = self.clf
        dic_y_mapping = {n: label for n, label in
                         enumerate(np.unique(y_train))}
        inverse_dic = {v: k for k, v in dic_y_mapping.items()}
        y_train_tag = np.array([inverse_dic[y] for y in y_train])

        history = clf.fit(x=X_train, y=y_train_tag, batch_size=256,
                          epochs=20, shuffle=True, verbose=0)

        self.history = history

        predicted_prob = self.clf.predict(X_test)
        predicted = [dic_y_mapping[np.argmax(pred)] for pred in
                     predicted_prob]
        return predicted, predicted_prob

class XGBClfCVFolds(CVFoldsModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.clf = xgb.XGBClassifier()
        self.name = "XGB" + self.name

        params = {"learning_rate":0.05, "max_depth":6, "n_estimators":800}
        self.clf.set_params(**params)

class RFClfCVFolds(CVFoldsModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.clf = RandomForestClassifier(max_depth=50, min_samples_split=2, min_samples_leaf=1, n_estimators=1000)
        self.name = "RF" + self.name

class SVMClfCVFolds(CVFoldsModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.clf = SVC(kernel="rbf", C=1, gamma='auto', probability=True)
        self.name = "SVM" + self.name


import logging
import os
import sys
from src.genomic_embeddings.models import NNClfCVFolds, RFClfCVFolds, XGBClfCVFolds, SVMClfCVFolds
from src.genomic_embeddings.plot import FoldModelPlots
import argparse
import pickle


argparse = argparse.ArgumentParser()
argparse.add_argument('--cv', default='LOPOCV', type=str, help='name of the cross validation')
argparse.add_argument('--output',
                      default='/predictions',
                      type=str, help='predictions output dir')
argparse.add_argument('--metadata',
                      default='metadata.csv',
                      type=str, help='metadata csv file path')
argparse.add_argument('--fold2data',
                      default='fold2data.pkl',
                      type=str, help='mapping of LOPOCV train and test')

params = argparse.parse_args()

CV = params.cv
METADATA = params.metadata
OUTPUT_DIR = params.output
FOLD2DATA = params.fold2data

# configure logger
logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s',
                    filename=os.path.join(OUTPUT_DIR, f"Validation.log"), level=logging.INFO)

# top predicted LABEL
curated_labels_no_pumps = ['Amino sugar and nucleotide sugar metabolism',
                           'Benzoate degradation',
                           'Energy metabolism',
                           'Oxidative phosphorylation',
                           'Porphyrin and chlorophyll metabolism',
                           'Prokaryotic defense system',
                           'Ribosome',
                           'Secretion system',
                           'Two-component system']

labels = [(curated_labels_no_pumps, 'NO-PUMPS-CURATED-LABELS')]
LABEL = 'label'
q = ''

with open(FOLD2DATA, 'rb') as o:
    fold2data = pickle.load(o)

for label, label_alias in labels:
    alias = label_alias
    MDLS = [(NNClfCVFolds(X=1, y=1, cv=5, out_dir=OUTPUT_DIR, fold2data=fold2data[CV], fold_type=CV), 'CVFOLD_' + alias),
            (XGBClfCVFolds(X=1, y=1, cv=5, out_dir=OUTPUT_DIR, fold2data=fold2data[CV], fold_type=CV), 'CVFOLD_' + alias),
            (RFClfCVFolds(X=1, y=1, cv=5, out_dir=OUTPUT_DIR, fold2data=fold2data[CV], fold_type=CV), 'CVFOLD_' + alias),
            (SVMClfCVFolds(X=1, y=1, cv=5, out_dir=OUTPUT_DIR, fold2data=fold2data[CV], fold_type=CV), 'CVFOLD_' + alias)]


    for mdl, name in MDLS:
        mdl.classification_pipeline(LABEL, alias=name)
        plotter = FoldModelPlots(mdl=mdl)
        plotter.plot_single_aupr_with_ci()
        plotter.plot_single_roc_with_ci()
        plotter.plot_precision_recall()
        plotter.plot_roc()
        plotter.plot_precision_recall_by_fold()
        plotter.plot_roc_by_fold()